@@ -1,17 +1,27 @@
 import os
 import sys
+import codecs
 from itertools import chain
 
 from .types import convert_type, BOOL
-from .utils import echo
+from .utils import make_str, make_default_short_help
 from .exceptions import UsageError, Abort
-from .helpers import prompt, confirm
+from .helpers import prompt, confirm, echo
+from .formatting import HelpFormatter
+from .parser import OptionParser
 
-from . import _optparse
+from ._compat import PY2, isidentifier
 
 _missing = object()
 
 
+def _cleanup_name(name):
+    prefix = name[:1]
+    if not prefix.isalnum():
+        return name.lstrip(prefix)
+    return name
+
+
 class Context(object):
     """The context is a special internal object that holds state relevant
     for the script execution at every single level.  It's normally invisible
@@ -76,7 +86,6 @@ class Context(object):
             self.auto_envvar_prefix = auto_envvar_prefix.upper()
         self.auto_envvar_prefix = auto_envvar_prefix
 
-        self._parser = command._make_parser(self)
         self._close_callbacks = []
 
     def __enter__(self):
@@ -131,6 +140,15 @@ class Context(object):
                 return node.obj
             node = node.parent
 
+    def ensure_object(self, object_type):
+        """Like :meth:`find_object` but sets the innermost object to a
+        new instance of `object_type` if it does not exist.
+        """
+        rv = self.find_object(object_type)
+        if rv is None:
+            self.obj = rv = object_type()
+        return rv
+
     def fail(self, message):
         """Aborts the execution of the program with a specific error
         message.
@@ -147,27 +165,55 @@ class Context(object):
         """Exits the application with a given exit code."""
         sys.exit(code)
 
-    def format_usage(self):
+    def get_usage(self):
         """Helper method to get formatted usage string for the current
         context and command.
         """
-        return self._parser.get_usage()
+        return self.command.get_usage(self)
 
-    def format_help(self):
+    def get_help(self):
         """Helper method to get formatted help page for the current
         context and command.
         """
-        return self._parser.format_help().rstrip()
+        return self.command.get_help(self)
 
     def invoke(*args, **kwargs):
         """Invokes a command callback in exactly the way it expects.
         """
         self, callback = args[:2]
+
+        # It's also possible to invoke another command which might or
+        # might not have a callback.
+        if isinstance(callback, Command):
+            callback = callback.callback
+            if callback is None:
+                raise TypeError('The given command does not have a '
+                                'callback that can be invoked.')
+
         args = args[2:]
         if getattr(callback, '__click_pass_context__', False):
             args = (self,) + args
         return callback(*args, **kwargs)
 
+    def forward(*args, **kwargs):
+        """Similar to :meth:`forward` but fills in default keyword
+        arguments from the current context if the other command expects
+        it.  This cannot invoke callbacks directly, only other commands.
+        """
+        self, cmd = args[:2]
+
+        # It's also possible to invoke another command which might or
+        # might not have a callback.
+        if not isinstance(cmd, Command):
+            raise TypeError('Callback is not a command.')
+
+        for param_name in cmd.iter_param_names():
+            if param_name in self.params and \
+               param_name not in kwargs:
+                kwargs[param_name] = self.params[param_name]
+
+        return self.invoke(cmd, **kwargs)
+
 
 class Command(object):
     """Commands are the basic building block of command line interfaces in
@@ -207,11 +253,7 @@ class Command(object):
         self.epilog = epilog
         self.options_metavar = options_metavar
         if short_help is None and help:
-            short_help = help.split('.')[0].strip()
-            if len(short_help) > 45:
-                short_help = None
-            elif short_help:
-                short_help = short_help + '.'
+            short_help = make_default_short_help(help)
         self.short_help = short_help
         if add_help_option:
             help_option()(self)
@@ -228,15 +270,82 @@ class Command(object):
             if not param.is_eager:
                 yield param
 
-    def _make_parser(self, ctx):
-        parser = _optparse._SimplifiedOptionParser(
-            ctx, description=self.help, epilog=self.epilog)
+    def iter_param_names(self):
+        """Iterates over all parameter names."""
+        for param in self.iter_params_for_processing():
+            yield param.name
+
+    def make_parser(self, ctx):
+        """Creates the underlying option parser for this command."""
+        parser = OptionParser(ctx)
         for param in self.params:
-            param._add_to_parser(parser, ctx)
+            param.add_to_parser(parser, ctx)
         return parser
 
-    def _format_extra_help(self, ctx):
-        pass
+    def get_usage(self, ctx):
+        """Formats the usage line into a string and returns it.  This
+        creates a formatter and will call into the following formatting
+        methods:
+
+        -   :meth:`format_usage`
+        """
+        formatter = HelpFormatter()
+        self.format_usage(ctx, formatter)
+        return formatter.getvalue().rstrip('\n')
+
+    def get_help(self, ctx):
+        """Formats the help into a string and returns it.  This creates a
+        formatter and will call into the following formatting methods:
+
+        -   :meth:`format_usage`
+        -   :meth:`format_help`
+        -   :meth:`format_options`
+        -   :meth:`format_epilog`
+        """
+        formatter = HelpFormatter()
+        self.format_usage(ctx, formatter)
+        self.format_help(ctx, formatter)
+        self.format_options(ctx, formatter)
+        self.format_epilog(ctx, formatter)
+        return formatter.getvalue().rstrip('\n')
+
+    def format_usage(self, ctx, formatter):
+        """Writes the usage line into the formatter."""
+        pieces = self.collect_usage_pieces(ctx)
+        formatter.write_usage(ctx.command_path, ' '.join(pieces))
+
+    def format_help(self, ctx, formatter):
+        """Writes the help into the formatter if it exists."""
+        if self.help:
+            formatter.write_paragraph()
+            with formatter.indentation():
+                formatter.write_text(self.help)
+
+    def format_options(self, ctx, formatter):
+        """Writes all the options into the formatter if they exist."""
+        opts = []
+        for param in self.params:
+            rv = param.get_help_record(ctx)
+            if rv is not None:
+                opts.append(rv)
+
+        if opts:
+            with formatter.section('Options'):
+                formatter.write_dl(opts)
+
+    def format_epilog(self, ctx, formatter):
+        """Writes the epilog into the formatter if it exists."""
+        if self.epilog:
+            formatter.write_paragraph()
+            with formatter.indentation():
+                formatter.write_text(self.epilog)
+
+    def collect_usage_pieces(self, ctx):
+        """Returns all the pieces that go into the usage line."""
+        rv = [self.options_metavar]
+        for param in self.params:
+            rv.extend(param.get_usage_pieces(ctx))
+        return rv
 
     def make_context(self, info_name, args, parent=None, **extra):
         """This function when given an info name and arguments will kick
@@ -254,15 +363,16 @@ class Command(object):
                       constructor.
         """
         ctx = Context(self, info_name=info_name, parent=parent, **extra)
-        opts, args = ctx._parser.parse_args(args=args)
-        opts = opts.__dict__
+        parser = self.make_parser(ctx)
+        opts, args = parser.parse_args(args=args)
 
         for param in ctx.command.iter_params_for_processing():
             value, args = param.handle_parse_result(ctx, opts, args)
 
         if args and not self.allow_extra_args:
             ctx.fail('Got unexpected extra argument%s (%s)'
-                     % (len(args) != 1 and 's' or '', ' '.join(args)))
+                     % (len(args) != 1 and 's' or '',
+                        ' '.join(map(make_str, args))))
 
         ctx.args = args
 
@@ -292,12 +402,30 @@ class Command(object):
         :param extra: extra keyword arguments are forwarded to the context
                       constructor.
         """
+        # If we are on python 3 we will verify that the environment is
+        # sane at this point of reject further execution to avoid a
+        # broken script.
+        if not PY2:
+            try:
+                import locale
+                fs_enc = codecs.lookup(locale.getpreferredencoding()).name
+            except Exception:
+                fs_enc = 'ascii'
+            if fs_enc == 'ascii':
+                raise RuntimeError('Click will abort further execution '
+                                   'because Python 3 was configured to use '
+                                   'ASCII as encoding for the environment. '
+                                   'Either switch to Python 2 or consult '
+                                   'http://click.pocoo.org/python3/ '
+                                   'for mitigation steps.')
+
         if args is None:
             args = sys.argv[1:]
         else:
             args = list(args)
         if prog_name is None:
-            prog_name = os.path.basename(sys.argv and sys.argv[0] or __file__)
+            prog_name = make_str(os.path.basename(
+                sys.argv and sys.argv[0] or __file__))
         try:
             try:
                 with self.make_context(prog_name, args, **extra) as ctx:
@@ -308,7 +436,7 @@ class Command(object):
                 raise Abort()
             except UsageError as e:
                 if e.ctx is not None:
-                    echo(e.ctx.format_usage(), file=sys.stderr)
+                    echo(e.ctx.get_usage() + '\n', file=sys.stderr)
                 echo('Error: %s' % e.message, file=sys.stderr)
                 sys.exit(2)
         except Abort:
@@ -349,40 +477,48 @@ class MultiCommand(Command):
         self.invoke_without_command = invoke_without_command
         self.subcommand_metavar = subcommand_metavar
 
-    def _make_parser(self, ctx):
-        parser = Command._make_parser(self, ctx)
-        parser.usage += ' ' + self.subcommand_metavar
-        parser.disable_interspersed_args()
+    def make_parser(self, ctx):
+        parser = Command.make_parser(self, ctx)
+        parser.allow_interspersed_args = False
         return parser
 
-    def _format_extra_help(self, ctx):
-        commands = self.list_commands(ctx)
-        if not commands:
-            return
+    def collect_usage_pieces(self, ctx):
+        rv = Command.collect_usage_pieces(self, ctx)
+        rv.append(self.subcommand_metavar)
+        return rv
 
-        longest = len(sorted(commands, key=len)[-1])
+    def format_options(self, ctx, formatter):
+        Command.format_options(self, ctx, formatter)
+        self.format_commands(ctx, formatter)
 
-        subcommand_info = []
+    def format_commands(self, ctx, formatter):
+        """Extra format methods for multi methods that adds all the commands
+        after the options.
+        """
+        rows = []
         for subcommand in self.list_commands(ctx):
             cmd = self.get_command(ctx, subcommand)
             # What is this, the tool lied about a command.  Ignore it
             if cmd is None:
                 continue
-            help = cmd.short_help or ''
-            subcommand_info.append('  %-*s  %s' % (longest, subcommand, help))
 
-        return 'Commands:\n%s' % '\n'.join(subcommand_info)
+            help = cmd.short_help or ''
+            rows.append((subcommand, help))
+
+        if rows:
+            with formatter.section('Commands'):
+                formatter.write_dl(rows)
 
     def invoke(self, ctx):
         if not ctx.args:
             if self.invoke_without_command:
                 return Command.invoke(self, ctx)
             elif self.no_args_is_help:
-                echo(ctx.format_help())
+                echo(ctx.get_help())
                 ctx.exit()
             ctx.fail('Missing command')
 
-        cmd_name = ctx.args[0]
+        cmd_name = make_str(ctx.args[0])
         cmd = self.get_command(ctx, cmd_name)
         if cmd is None:
             ctx.fail('No such command "%s"' % cmd_name)
@@ -443,6 +579,18 @@ class Group(MultiCommand):
             return cmd
         return decorator
 
+    def group(self, *args, **kwargs):
+        """A shortcut decorator for declaring and attaching a group to
+        the group.  This takes the same arguments as :func:`group` but
+        immediately registers the created command with this instance by
+        calling into :meth:`add_command`.
+        """
+        def decorator(f):
+            cmd = group(*args, **kwargs)(f)
+            self.add_command(cmd)
+            return cmd
+        return decorator
+
     def get_command(self, ctx, cmd_name):
         return self.commands.get(cmd_name)
 
@@ -450,6 +598,35 @@ class Group(MultiCommand):
         return sorted(self.commands)
 
 
+class CommandCollection(MultiCommand):
+    """A command collection is a multi command that merges multiple multi
+    commands together into one.  This is a straightforward implementation
+    that accepts a list of different multi commands as sources and
+    provides all the commands for each of them.
+    """
+
+    def __init__(self, name=None, sources=None, **attrs):
+        MultiCommand.__init__(self, name, **attrs)
+        #: The list of registered multi commands.
+        self.sources = sources or []
+
+    def add_source(self, multi_cmd):
+        """Adds a new multi command to the chain dispatcher."""
+        self.sources.append(multi_cmd)
+
+    def get_command(self, ctx, cmd_name):
+        for source in self.sources:
+            rv = source.get_command(ctx, cmd_name)
+            if rv is not None:
+                return rv
+
+    def list_commands(self, ctx):
+        rv = set()
+        for source in self.sources:
+            rv.update(source.list_commands(ctx))
+        return sorted(rv)
+
+
 class Parameter(object):
     """A parameter to a command comes in two versions: they are either
     :class:`Option`\s or :class:`Argument`\s.  Other subclasses are currently
@@ -468,7 +645,6 @@ class Parameter(object):
     :param default: the default value if omitted.  This can also be a callable
                     in which case it's invoked when the default is needed
                     without any arguments.
-    :param help: the help string.
     :param callback: a callback that should be executed after the parameter
                      was matched.  This is called as ``fn(ctx, value)`` and
                      needs to return the value.
@@ -487,14 +663,12 @@ class Parameter(object):
     param_type_name = 'parameter'
 
     def __init__(self, param_decls=None, type=None, required=False,
-                 default=None, help=None, callback=None, nargs=1,
-                 metavar=None, expose_value=True, is_eager=False,
-                 envvar=None):
+                 default=None, callback=None, nargs=1, metavar=None,
+                 expose_value=True, is_eager=False, envvar=None):
         self.name, self.opts, self.secondary_opts = \
             self._parse_decls(param_decls or ())
         self.type = convert_type(type, default)
         self.required = required
-        self.help = help
         self.callback = callback
         self.nargs = nargs
         self.multiple = False
@@ -524,8 +698,14 @@ class Parameter(object):
             rv = self.default
         return self.type(rv, self, ctx)
 
-    def _add_to_parser(self, parser, ctx):
-        raise NotImplementedError()
+    def add_to_parser(self, parser, ctx):
+        pass
+
+    def consume_value(self, ctx, opts):
+        value = opts.get(self.name)
+        if value is None:
+            value = self.value_from_envvar(ctx)
+        return value
 
     def process_value(self, ctx, value):
         """Given a value and context this runs the logic to convert the
@@ -549,9 +729,10 @@ class Parameter(object):
         return value
 
     def get_missing_message(self, ctx):
-        return 'Missing %s "%s".' % (
+        return 'Missing %s %s.' % (
             self.param_type_name,
-            ' / '.join(chain(self.opts, self.secondary_opts)),
+            ' / '.join('"%s"' % x for x in chain(
+                self.opts, self.secondary_opts)),
         )
 
     def value_from_envvar(self, ctx):
@@ -565,11 +746,8 @@ class Parameter(object):
         else:
             return os.environ.get(self.envvar)
 
-    def consume_value(self, ctx, opts, args):
-        return None, args
-
     def handle_parse_result(self, ctx, opts, args):
-        value, args = self.consume_value(ctx, opts, args)
+        value = self.consume_value(ctx, opts)
         value = self.full_process_value(ctx, value)
         if self.callback is not None:
             value = self.callback(ctx, value)
@@ -577,6 +755,12 @@ class Parameter(object):
             ctx.params[self.name] = value
         return value, args
 
+    def get_help_record(self, ctx):
+        pass
+
+    def get_usage_pieces(self, ctx):
+        return []
+
 
 class Option(Parameter):
     """Options are usually optionaly values on the command line and
@@ -607,6 +791,7 @@ class Option(Parameter):
                                parameter will be pulled from an environment
                                variable in case a prefix is defined on the
                                context.
+    :param help: the help string.
     """
     param_type_name = 'option'
 
@@ -614,7 +799,7 @@ class Option(Parameter):
                  prompt=False, confirmation_prompt=False,
                  hide_input=False, is_flag=None, flag_value=None,
                  multiple=False, allow_from_autoenv=True, type=None,
-                 **attrs):
+                 help=None, **attrs):
         default_is_missing = attrs.get('default', _missing) is _missing
         Parameter.__init__(self, param_decls, type=type, **attrs)
         if prompt is True:
@@ -654,6 +839,7 @@ class Option(Parameter):
             self.is_bool_flag = False
 
         self.allow_from_autoenv = allow_from_autoenv
+        self.help = help
 
         # Sanity check for stuff we don't support
         if __debug__:
@@ -673,31 +859,66 @@ class Option(Parameter):
         possible_names = []
 
         for decl in decls:
-            if not decl.startswith('-'):
+            if isidentifier(decl):
                 if name is not None:
                     raise TypeError('Name defined twice')
                 name = decl
             else:
                 if '/' in decl:
                     first, second = decl.split('/', 1)
-                    possible_names.append(first.lstrip('-'))
+                    possible_names.append(_cleanup_name(first))
                     opts.append(first)
                     secondary_opts.append(second)
                 else:
-                    possible_names.append(decl.lstrip('-'))
+                    possible_names.append(_cleanup_name(decl))
                     opts.append(decl)
 
         if name is None and possible_names:
             possible_names.sort(key=len)
-            name = possible_names[-1]
+            name = possible_names[-1].replace('-', '_')
 
         if name is None:
             raise TypeError('Could not determine name for option')
 
-        return name.replace('-', '_'), opts, secondary_opts
+        return name, opts, secondary_opts
 
-    def _add_to_parser(self, parser, ctx):
-        help = self.help
+    def add_to_parser(self, parser, ctx):
+        kwargs = {
+            'dest': self.name,
+            'nargs': self.nargs,
+        }
+
+        action = self.multiple and 'append' or 'store'
+
+        if self.is_flag:
+            kwargs.pop('nargs', None)
+            if self.is_bool_flag and self.secondary_opts:
+                parser.add_option(self.opts, action=action + '_const',
+                                  const=True, **kwargs)
+                kwargs.pop('default', None)
+                parser.add_option(self.secondary_opts, action=action +
+                                  '_const', const=False, **kwargs)
+            else:
+                parser.add_option(self.opts, action=action + '_const',
+                                  const=self.flag_value,
+                                  **kwargs)
+        else:
+            kwargs['action'] = action
+            parser.add_option(self.opts, **kwargs)
+
+    def get_help_record(self, ctx):
+        def _write_opts(opts):
+            rv = []
+            for opt in opts:
+                if not self.is_flag:
+                    opt += ' ' + self.make_metavar()
+                rv.append(opt)
+            return ', '.join(rv)
+        rv = [_write_opts(self.opts)]
+        if self.secondary_opts:
+            rv.append(_write_opts(self.secondary_opts))
+
+        help = self.help or ''
         extra = []
         if self.default is not None and self.show_default:
             extra.append('default: %s' % self.default)
@@ -708,35 +929,7 @@ class Option(Parameter):
         if extra:
             help = '%s[%s]' % (help and help + '  ' or '', '; '.join(extra))
 
-        kwargs = {
-            'help': help,
-            'dest': self.name,
-            'nargs': self.nargs,
-            'metavar': self.make_metavar(),
-        }
-
-        action = self.multiple and 'append' or 'store'
-
-        if self.is_flag:
-            kwargs.pop('nargs', None)
-            if self.is_bool_flag and self.secondary_opts:
-                pos_opt = _optparse.make_option(
-                    *self.opts, action=action + '_true', **kwargs)
-                kwargs.pop('default', None)
-                kwargs.pop('help', None)
-                neg_opt = _optparse.make_option(
-                    *self.secondary_opts, action=action + '_false',
-                    help=_optparse.SUPPRESS_HELP, **kwargs)
-                pos_opt._negative_version = neg_opt
-                parser.add_option(pos_opt)
-                parser.add_option(neg_opt)
-            else:
-                parser.add_option(*self.opts, action=action + '_const',
-                                  const=self.flag_value,
-                                  **kwargs)
-        else:
-            kwargs['action'] = action
-            parser.add_option(*self.opts, **kwargs)
+        return (' / '.join(rv), help)
 
     def get_default(self, ctx):
         # If we're a non boolean flag out default is more complex because
@@ -782,12 +975,6 @@ class Option(Parameter):
             return self.prompt_for_value(ctx)
         return Parameter.full_process_value(self, ctx, value)
 
-    def consume_value(self, ctx, opts, args):
-        value = opts.get(self.name)
-        if value is None:
-            value = self.value_from_envvar(ctx)
-        return value, args
-
 
 class Argument(Parameter):
     """Arguments are positional parameters to a command.  They generally
@@ -816,38 +1003,20 @@ class Argument(Parameter):
             raise TypeError('Could not determine name for argument')
         if len(decls) == 1:
             name = arg = decls[0]
+            name = name.replace('-', '_')
         elif len(decls) == 2:
             name, arg = decls
         else:
             raise TypeError('Arguments take exactly one or two '
                             'parameter declarations, got %d' % len(decls))
-        return name.replace('-', '_'), [arg], []
+        return name, [arg], []
 
-    def _add_to_parser(self, parser, ctx):
-        parser.usage += ' ' + self.make_metavar()
+    def get_usage_pieces(self, ctx):
+        return [self.make_metavar()]
 
-    def consume_value(self, ctx, opts, args):
-        found = True
-        if self.nargs == 1:
-            try:
-                value = args.pop(0)
-            except IndexError:
-                found = False
-        elif self.nargs < 0:
-            value = tuple(args)
-            found = value
-            args = []
-        else:
-            values = args[:self.nargs]
-            args = args[self.nargs:]
-            value = tuple(values)
-            found = value
-        if not found:
-            value = self.value_from_envvar(ctx)
-            if self.nargs != 1:
-                value = value and (value,) or ()
-        return value, args
+    def add_to_parser(self, parser, ctx):
+        parser.add_argument(dest=self.name, nargs=self.nargs)
 
 
 # Circular dependency between decorators and core
-from .decorators import command, help_option
+from .decorators import command, group, help_option
